package uulm.teamname.marvelous.server.lobbymanager;

import org.java_websocket.WebSocket;
import uulm.teamname.marvelous.gamelibrary.messages.ParticipantType;

import java.util.Objects;

public class Participant {

    /** The Websocket to contact the participant with */
    private WebSocket connection;

    /** The type (as in role) of participant */
    public final ParticipantType type;

    /** Whether the participant is an AI */
    public final boolean AI;

    /** Creates a new {@link Participant} */
    public Participant (WebSocket connection, ParticipantType type, boolean AI) {
        this.connection = connection;
        this.type = type;
        this.AI = AI;
    }

    /** Returns the {@link WebSocket} to contact the participant with */
    public WebSocket getConnection() {
        return connection;
    }

    /** Sets the connection {@link WebSocket} for the current participant */
    public void setConnection(WebSocket connection) {
        this.connection = connection;
    }

    /** Removes reference to current connection from Participant */
    public void clearConnection() {
        this.connection = null;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Participant that = (Participant) o;
        return AI == that.AI && Objects.equals(connection, that.connection) && type == that.type;
    }

    @Override
    public int hashCode() {
        return Objects.hash(connection, type, AI);
    }

    @Override
    public String toString() {
        return "Participant{" +
                "connection=" + connection +
                ", type=" + type +
                ", AI=" + AI +
                '}';
    }
}
