package uulm.teamname.marvelous.server.netconnector;

import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.handshake.ClientHandshakeBuilder;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import uulm.teamname.marvelous.gamelibrary.json.JSON;
import uulm.teamname.marvelous.gamelibrary.messages.ParticipantType;
import uulm.teamname.marvelous.gamelibrary.messages.client.*;
import uulm.teamname.marvelous.server.lobbymanager.Participant;

import java.util.HashMap;
import java.util.Iterator;

import static org.mockito.Mockito.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.assertj.core.api.Assertions.*;

class UserManagerTest {

    UserManager manager;
    WebSocket connection;
    Participant participant;
    ClientHandshake handshake;

    @BeforeEach
    void beforeEach() {
        connection = mock(WebSocket.class);
        // when(connection.hashCode()).thenReturn(1); // just for testing, of course
        manager = spy(new UserManager(new JSON(null))); // FIXME: This shouldn't be null, of course

        handshake = mock(ClientHandshake.class);
        when(handshake.getResourceDescriptor()).thenReturn("/someDescriptor");

        participant = spy(new Participant(connection, ParticipantType.PlayerOne, false));
    }

    @Test
    void userIsConnectedTest() {
        assertThat(manager.getUserCount()).isEqualTo(0);
        assertThat(manager.getNewUsers()).doesNotContain(connection);
        manager.connectUser(connection, handshake);
        assertThat(manager.getUserCount()).isEqualTo(1);
        assertThat(manager.isUserConnected(connection)).isTrue();
        assertThat(manager.getNewUsers()).containsOnly(connection);
    }

    @Test
    void helloServerMessagesGetAssignedProperly() {
        manager.messageReceived(
                connection,
                "{\"messageType\":\"HELLO_SERVER\",\"name\":\"SomeAwesomeName\",\"deviceID\":\"YAY\"}");

        verify(manager).handshake(
                eq(connection),
                any(HelloServerMessage.class));
    }

    @Test
    void handshakeGetsCompletedProperlyTest() {
        manager.connectUser(connection, handshake);
        assertThat(manager.getUserCount()).isEqualTo(1);
        assertThat(manager.getNewUsers()).containsOnly(connection);
        assertThat(manager.getReadyToConnect()).isEmpty();

        var message = new HelloServerMessage();
        message.name = "Some Awesome Name";
        message.deviceID = "Some Interesting ID";

        manager.handshake(connection, message);

        assertThat(manager.getUserCount()).isEqualTo(1);
        assertThat(manager.getNewUsers()).isEmpty();
        assertThat(manager.getReadyToConnect()).containsOnlyKeys(connection);

    }

    void reconnectMessagesGetAssignedProperly() {
        manager.messageReceived(
                connection,
                "{\"messageType\":\"RECONNECT\",\"reconnect\":\"TRUE\"}");

        verify(manager).reconnectClient(
                eq(connection),
                any(ReconnectMessage.class));
    }

    @Test
    @Disabled
    void reconnectWorksProperly() {
        // TODO: create test for this case
    }

    void playerReadyMessagesGetAssignedProperly() {
        manager.messageReceived(
                connection,
                "{\"messageType\":\"PLAYER_READY\",\"startGame\":\"false\",\"role\":\"Player\"}");

        verify(manager).assignLobby(
                eq(connection),
                any(PlayerReadyMessage.class));
    }


    void characterSelectionMessageGetsAssignedProperly() {
        manager.messageReceived(
                connection,
                "{\"messageType\":\"CHARACTER_SELECTION\",\"characters\":" +
                        "[true, true, false, true, false, false, true, true, false, false, false, true]}");

        verify(manager).charactersSelected(
                eq(connection),
                any(CharacterSelectionMessage.class));
    }

    void requestMessagesGetRelayedProperly() {
        manager.messageReceived(
                connection,
                "{\"messageType\":\"REQUESTS\",\"messages\":[]}");

        verify(manager).relayRequestMessage(
                eq(participant),
                any(RequestMessage.class));
    }
}
